const fs = require('fs');
const path = require('path');
const glob = require('tiny-glob/sync');

const mkdirp = dir => {
	try {
		fs.mkdirSync(dir, { recursive: true });
	} catch {}
}

module.exports = function builder({
	input,
	output,
	manifest
}, opts = {}) {
	mkdirp(`${output}/assets/_app`);

	// copy client code
	const client_code = path.resolve(input, 'client');
	glob('**/*', { cwd: client_code, filesOnly: true }).forEach(file => {
		if (file[0] !== '.') {
			fs.copyFileSync(`${client_code}/${file}`, `${output}/assets/_app/${file}`);
		}
	});

	// copy server code
	const server_code = path.resolve(input, 'server');
	glob('**/*', { cwd: server_code, filesOnly: true }).forEach(file => {
		if (file[0] !== '.') {
			mkdirp(path.dirname(`${output}/${file}`));
			fs.copyFileSync(`${server_code}/${file}`, `${output}/${file}`);
		}
	});

	// copy the server entry point
	fs.copyFileSync(path.resolve(__dirname, 'server.js'), `${output}/index.js`);

	// generate manifest
	const written_manifest = `module.exports = {
		layout: ${JSON.stringify(manifest.layout)},
		error: ${JSON.stringify(manifest.error)},
		components: ${JSON.stringify(manifest.components)},
		pages: [
			${manifest.pages.map(page => `{ pattern: ${page.pattern}, parts: ${JSON.stringify(page.parts)} }`).join(',\n\t\t\t')}
		],
		server_routes: [
			${manifest.server_routes.map(route => `{ name: '${route.name}', pattern: ${route.pattern}, file: '${route.file}', params: ${JSON.stringify(route.params)} }`).join(',\n\t\t\t')}
		]
	};`.replace(/^\t/gm, '');

	fs.writeFileSync(`${output}/manifest.js`, written_manifest);

	// copy client manifest
	fs.copyFileSync(`${input}/client.json`, `${output}/client.json`);

	// copy template
	fs.copyFileSync(`src/app.html`, `${output}/app.html`);
};